import sys
import os

sys.path.append(os.path.dirname(__file__) + os.sep + '../')

from s3.minio.minio import YcMinio
from dataset.request import Request

# 约定：dataset下的所有方法均是对数据集的操作，非clint调用的地方，严禁出现bucket之类的存储对象用语
# 本类用来屏蔽各种存储对象(oss, minio, 七牛)之间的差异，通过type构造不同的客户端解决问题，类似工厂类
# 各个OSS均需要实现本类中的所有方法
# bucket = dataset
# object = record
class Dataset():
    def __init__(self, T_key, datasetName, ip):
        self.req = Request(T_key, datasetName, ip)
        info = self.req.GetAccess()
        self.access_key = info['access_key']
        self.secret_key = info['secret_key']
        self.endpoint = info['endpoint']
        self.bucket = info['bucket']
        self.type = info['type']
        self.datasetName = datasetName
        self.ip = ip
        if self.type == "minio":
            self.client = YcMinio(self.access_key, self.secret_key, self.endpoint)
            if datasetName not in self.ListAllDataset():
                raise Exception("can not connect to the dataset, please make sure you have the correct access rights and the dataset exists!")
        else:
            raise Exception("尚未支持其他类型的云存储")


    #列出所有数据集
    def ListAllDataset(self, datasetName="", recu=False):
        ret = [dataset.replace("/", "") for dataset in self.client.ListObjects(self.bucket)]
        return ret

    #列出谋个数据集的所有文件
    def ListAllFile(self):
        ret = [file for file in self.client.ListObjects(self.bucket, prefix=self.datasetName, recursive=True)]
        return ret

    #直传文件
    def PutFilesToDataset(self, datasetName, rootPath, ext="", batchNum=0):
        total = 0        
        i = 0
        for root, dir, files in os.walk(rootPath):
            for fileName in files:
                if fileName.endswith(ext):
                    total += 1

        dirname = os.path.dirname(rootPath.rstrip("/"))
        for root, dir, files in os.walk(rootPath):
            for fileName in files:
                if fileName.endswith(ext):
                    filePath = os.path.join(root, fileName)
                    objectName = filePath.replace(dirname, "")
                    self.PutFileToDataset(self.bucket, datasetName + objectName, filePath)
                    i += 1
                    per = (i * 100) // total
                    showText = filePath + " =====> " + datasetName + objectName
                    process = "\r[%3s%%]: %s" % (per, showText)
                    print(process)

        print("success total:" + str(total))
        return True
    
    def PutFileToDataset(self, objectName, filePath):
        self.client.PutObject(self.bucket, self.datasetName + "/" + objectName, filePath)

    def DeleteFromDataset(self, objectName):
        self.client.DeleteObject(self.bucket, self.datasetName + "/" + objectName)
        return True

    def SyncDataToWeb(self, data):
        info = self.req.Upload(data)
        return info

    def Delete(self, referId):
        info = self.req.Delete(referId)
        #todo 删除oss文件
        # self.DeleteFromDataset(self.datasetName, info["osspath"])
        return info

    def GetData(self, offset, limit):
        info = self.req.GetData(offset, limit)
        for item in info["items"]:
            item['accessUrl'] = self.client.PresignedGetObject(self.bucket, item["path"], expire=7)

        return info






import click
import os
import sys
import json
from minio import Minio

sys.path.append(os.path.dirname(__file__) + os.sep + '../')

from client import cfg
from dataset.dataset import Dataset

#组入口
@click.group()
@click.option("-ak", "--access_key", 'access_key', default="", help="The accessKey of ycdata.")
@click.option("-sk", "--secret_key", 'secret_key', default="", help="The secretKey of ycdata.")
@click.option("-en", "--endpoint", "endpoint", default="http://127.0.0.1:8888/", help="The endpoint of ycdata.")
@click.option('--debug/--no-debug', default=False)
@click.pass_context
def main(ctx, access_key, secret_key, endpoint, debug):
    """ 云测数据数据集平台工具 """
    obj = {}
    if access_key != "" and secret_key != "":#按照临时设置执行
        obj = {
            "access_key":access_key,
            "secret_key":secret_key,
            "endpoint":endpoint,
            "DEBUG":debug
        }
    else:#如果已经设置conf 则按照设置的来
        if cfg._check():
            conf = cfg._getConf()
            obj = {
                "access_key":conf['access_key'],
                "secret_key":conf['secret_key'],
                "endpoint":conf['endpoint'],
                "DEBUG":debug
            }

    ctx.obj = Dataset(obj["access_key"], obj["secret_key"], obj["endpoint"])


#信息设置
@main.command()
@click.option("--access_key", 'access_key', default="", help="The accessKey of ycdata.")
@click.option("--secret_key", 'secret_key', default="", help="The secretKey of ycdata.")
@click.option("--endpoint", 'endpoint', default="http://127.0.0.1:8888/",  help="The secretKey of ycdata.")
@click.pass_context
def config(ctx, access_key, secret_key, endpoint):
    """ ycdata用户信息设置 """
    configFile = cfg._config_filepath()
    if access_key == "" or secret_key == "":
        if cfg._check():
            print(cfg._getConf())
    else:
        conf = {
            "access_key":access_key,
            "secret_key":secret_key,
            "endpoint":endpoint,
        }
        with open(configFile, "w") as config:
            json.dump(conf, config)
            print(conf)

#列出所有数据集
@main.command()
@click.option("--dataset", 'dataset', default="", help="数据集名称")
@click.option("--recursive", 'recursive', type=bool, default=False, help="是否使用递归查询")
@click.pass_context
def ls(ctx, dataset, recursive):
    """ 查看数据 """
    ctx.obj.ListAllDataset(dataset, recursive)


#创建数据集
@main.command()
@click.option("--dataset", 'dataset', default="", help="数据集名称")
@click.pass_context
def create(ctx, dataset):
    """ 创建数据集 """
    if dataset == "":
        click.echo("请输入数据集名称")
        exit()
    ctx.obj.CreateDataset(dataset)

@main.command()
@click.option("--dataset", 'dataset', default="", help="数据集名称")
@click.option("--dir", 'dir', default="", help="上传的文件夹名称")
@click.pass_context
def put(ctx, dataset, dir):
    """ 上传文件到数据集 """
    if dataset == "":
        click.echo("请输入数据集名称")
        exit()
    
    if dir == "":
        click.echo("请输入上传文件夹的名称")
        exit()

    ctx.obj.PutFilesToDataset(dataset, dir)

@main.command()
@click.option("--dataset", 'dataset', default="", help="数据集名称")
@click.option("--file", 'file', default="", help="要删除的完整路径")
@click.pass_context
def delete(ctx, dataset, file):
    """ 上传文件到数据集 """
    if dataset == "":
        click.echo("请输入数据集名称")
        exit()
    
    if file == "":
        click.echo("请输入要删除的文件路径")
        exit()

    ctx.obj.DeleteFromDataset(dataset, file)
    

if __name__ == '__main__':
    main(obj={})






import sys
import os
import requests
import json

sys.path.append(os.path.dirname(__file__) + os.sep + '../')


class Request:
    def __init__(self, T_key, dataset_id, ip):
        self.T_key = T_key
        self.dataset_id = dataset_id
        self.ip = ""
        if ip:
            self.ip = ip
            self.API_ADDRESS_PREFIX = f"http://{self.ip}/api/v1/"
        else:
            self.API_ADDRESS_PREFIX = "http://127.0.0.1/api/v1/"

    def RiseException(self, response_text):
        raise Exception("error msg from server: " + response_text)

    def GetAccess(self):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/getosskey'
        headers = {"T-Key":self.T_key}
        res = requests.request("GET", api, headers=headers)
        resData = json.loads(res.text)
        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "access_key":resData["data"]["access_key"],
            "secret_key":resData["data"]["access_token"],
            "endpoint":resData["data"]["endpoint"],
            "bucket":resData["data"]["bucket"],
            "type":resData["data"]["type"],
        }

        return ret

    def Upload(self, data):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/upload'
        headers = {"T-Key": self.T_key, "Content-Type":"application/json"}

        res = requests.post(api, headers=headers, data=json.dumps(data))
        resData = json.loads(res.text)
        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "succ": resData["data"]["succ"],
            "fail": resData["data"]["fail"],
        }

        return ret

    def Delete(self, referId):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/refloc/{referId}'
        headers = {"T-Key": self.T_key, "Content-Type": "application/json"}
        res = requests.delete(api, headers=headers)
        resData = json.loads(res.text)
        if resData['code'] != "":
            self.RiseException(res.text)

        ret = {
            "succ": resData["data"]["succ"],
            "fail": resData["data"]["fail"],
        }

        return ret

    def GetData(self, offset, limit):
        api = f'{self.API_ADDRESS_PREFIX}dataset/{self.dataset_id}/data'
        headers = {"T-Key": self.T_key, "Content-Type": "application/json"}
        res = requests.post(api, headers=headers, data=json.dumps({"offset":offset, "limit":limit}))
        resData = json.loads(res.text)
        if resData['code'] != "":
            self.RiseException(res.text)


        return resData["data"]



import json

class LabelData():
    def __init__(self):
        self.labels = []

    def AddLabels(self, label="", instance="", attrs={}, type="", data={}):
        label = {
            "label":label,
            "instance":instance,
            "type":type,
            "attrs":attrs,
            "data":data,
        }
        self.labels.append(label)

    def ToString(self):
        return json.dumps(self.labels)




import os, sys
import hashlib

sys.path.append(os.path.dirname(__file__) + os.sep + './')

from metadata import MetaData
from labeldata import LabelData
from utils import util

class File():
    def __init__(self, filepath, basepath, datasetName):
        self.metadata = {}
        self.labeldata = LabelData()
        self.datasetName = datasetName
        if basepath == "":
            self.objectPath = os.path.basename(filepath)
            self.osspath = datasetName + "/" + os.path.basename(filepath)
        else:
            basepath = basepath.rstrip("/")
            if filepath.find(basepath) < 0:
                raise Exception(f"basepath must be part of filepath!")
            self.osspath = datasetName + "/" + filepath.split(basepath + "/")[1]
            self.objectPath = filepath.split(basepath + "/")[1]

        self.referId = ""
        self.filepath = filepath
        self.filename = os.path.basename(filepath)
        self.md5 = util.getFileMd5(self.filepath)
        self.filesize = int(util.getFileSize(self.filepath))
        self.type = util.getFiletype(self.filepath)

        with open(self.filepath, "rb") as bf:
            file_md5 = hashlib.md5(bf.read())
            self.md5 = file_md5.hexdigest()

    #数据自查
    def SelfCheck(self):
        if not self.referId:
            raise Exception(f"referid must be setted!")


    def AddmetaData(self, madedata):
        self.metadata = MetaData(madedata)

    def AddReferId(self, ref_id):
        if ref_id == "":
            raise Exception(f"referId can not be empty string!")
        self.referId = ref_id

    #添加box
    def AddBox2D(self, box2d, label="", instance="", attrs={}):
        # 数据格式检查
        if not type(box2d) is dict:
            raise Exception(f"box must be a dict, {type(box2d)} gavin")

        keys = list(set(box2d.keys()))
        keys.sort()
        if keys != ['height', 'width', 'x', 'y']:
            raise Exception(f"box keys must be ['height', 'width', 'x', 'y'], {type(box2d)} gavin")

        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="box2d", data=box2d)

    #添加椭圆
    def AddEllipse(self, ellipse, label="", instance="", attrs={}):
        if not type(ellipse) is dict:
            raise Exception(f"box must be a dict, {type(ellipse)} gavin")

        keys = list(set(ellipse.keys()))
        keys.sort()
        if keys != ['height', 'width', 'x', 'y']:
            raise Exception(f"ellipse keys must be ['height', 'width', 'x', 'y'], {type(ellipse)} gavin")

        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="ellipse", data=ellipse)

    def AddPolygon(self, polygon, label="", instance="", attrs={}):
        if not type(polygon) is list:
            raise Exception(f"polygon must be a list, {type(polygon)} gavin")

        for poly in polygon:
            keys = list(set(poly.keys()))
            if keys != ["x", "y"]:
                raise Exception(f"poly point keys must be ['x', 'y'], {poly} gavin")


        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="polygon", data=polygon)


    def AddLine(self, line, label="", instance="", attrs={}):
        if not type(line) is list:
            raise Exception(f"polygon must be a list, {type(line)} gavin")

        for point in line:
            keys = list(set(point.keys()))
            if keys != ["x", "y"]:
                raise Exception(f"line point keys must be ['x', 'y'], {point} gavin")


        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="line", data=point)


    def AddCurve(self, curve, label="", instance="", attrs={}):
        if not type(curve) is list:
            raise Exception(f"polygon must be a list, {type(curve)} gavin")

        for point in curve:
            keys = list(set(point.keys()))
            if keys != ["x", "y"]:
                raise Exception(f"curve point keys must be ['x', 'y'], {point} gavin")


        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="curve", data=curve)

    def AddPoint(self, point, label="", instance="", attrs={}):
        if not type(point) is dict:
            raise Exception(f"curve must be a list, {type(point)} gavin")

        keys = list(set(point.keys()))
        if keys != ["x", "y"]:
            raise Exception(f"curve point keys must be ['x', 'y'], {point} gavin")


        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="poit", data=point)


    def AddParalle(self, paralle, label="", instance="", attrs={}):
        if not type(paralle) is list:
            raise Exception(f"paralle must be a list, {type(paralle)} gavin")

        if len(paralle) != 4:
            raise Exception(f"paralle points numbers must be 4, {len(paralle)} gavin")

        for point in paralle:
            keys = list(set(point.keys()))
            if keys != ["x", "y"]:
                raise Exception(f"paralle point keys must be ['x', 'y'], {point} gavin")


        self.labeldata.AddLabels(label=label, instance=instance, attrs=attrs, type="paralle", data=paralle)






import sys, os
import sqlite3

sys.path.append(os.path.dirname(__file__) + os.sep + './')

from utils import util
from file import File

class LocalDb():
    def __init__(self, commitId=""):
        self.commitId = commitId

        self.newDbFlag = False
        if self.commitId  == "":#新db
            self.commitId = util.getRandomSet(32)
            self.newDbFlag = True



        home = "USERPROFILE" if os.name == "nt" else "HOME"
        configDir = os.path.join(os.environ[home], ".tda/")
        if not os.path.exists(configDir):
            os.makedirs(configDir)

        self.dbDir = os.path.join(configDir, self.commitId)
        if not os.path.exists(self.dbDir):
            os.makedirs(self.dbDir)

        self.dbFile = os.path.join(self.dbDir, self.commitId + ".db")
        if not os.path.exists(self.dbFile) and not self.newDbFlag:
            raise Exception(f"commit db [{self.commitId}] not exist, have you ever commited them?")

        if not os.path.exists(self.dbFile) and self.newDbFlag:
            self.initDb()

    def initDb(self):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()

        if self.newDbFlag:
            createSql = '''CREATE TABLE TDADATA
                   (filepath CHAR(1024) PRIMARY KEY NOT NULL,
                    osspath CHAR(1024) NOT NULL,
                    objectPath CHAR(1024) NOT NULL,
                    filename CHAR(521) NOT NULL,
                    md5 CHAR(32) NOT NULL,
                    referid CHAR(128) NOT NULL,
                    filesize INT NOT NULL,
                    metadata CHAR(1024) NOT NULL,
                    labeldata BLOB NOT NULL
                   );'''
            self.cur.execute(createSql)

        self.conn.close()

    def close(self):
        self.conn.close()

    def getDataByFilepath(self, filepath):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()
        sql = f"SELECT * FROM TDADATA WHERE filepath = '{filepath}';"
        res = self.cur.execute(sql)
        return res


    def insertVal(self, data = []):
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()

        for file in data:
            file.SelfCheck()
            sql = f"SELECT * FROM TDADATA WHERE filepath = '{file.filepath}';"
            self.cur.execute(sql)
            res = self.cur.fetchone()
            if res != None:
                print(file.filepath + " already commited, continue! you can updload them whenever you want.")
                continue

            sql = f"INSERT INTO TDADATA (filepath, osspath, objectPath, filename, md5, referid, filesize, metadata, labeldata) VALUES ('{file.filepath}', '{file.osspath}', '{file.objectPath}', '{file.filename}', '{file.md5}', '{file.referId}', '{file.filesize}', '{file.metadata.ToString()}', '{file.labeldata.ToString()}')"

            self.cur.execute(sql)

        self.conn.commit()
        self.conn.close()


    def fetchAll(self):
        ret = []
        self.conn = sqlite3.connect(self.dbFile)
        self.cur = self.conn.cursor()
        sql = f"SELECT * FROM TDADATA;"
        res = self.cur.execute(sql)
        for row in res:
            # tmpFile = File()
            # tmpFile.filepath = row[0]
            # tmpFile.osspath = row[1]
            # tmpFile.filename = row[2]
            # tmpFile.md5 = row[3]
            # tmpFile.referid = row[4]
            # tmpFile.filesize = row[5]
            # tmpFile.metadata = row[6]
            # tmpFile.labeldata = row[7]

            tmp = {
                "filepath":row[0],
                "osspath":row[1],
                "objectPath":row[2],
                "filename":row[3],
                "md5":row[4],
                "referid":row[5],
                "filesize":row[6],
                "metadata":row[7],
                "labeldata":row[8],
            }

            ret.append(tmp)
        self.conn.close()
        return ret









from setuptools import setup, find_packages

setup(
    name = "testindatadev",
    version = "0.0.4",
    keywords = ["云测", "数据集","yuncedata", "testin", "testindatadev"],
    description = "数据集管理平台pythonSDK",
    long_description = "数据集管理平台pythonSDK，用于管理数据集",
    license = "MIT Licence",

    url = "http://ai.testin.cn/",
    author = "hide-in-code",
    author_email = "hejinlong@testin.cn",

    packages = find_packages(),
    entry_points={
          'console_scripts': [
              'tdav = testindatadev.cli.run:main'
          ]
      },
    include_package_data = True,
    platforms = "any",
    install_requires = [
        "click",
        "minio",
    ]
)



from minio import Minio
from minio.error import InvalidResponseError as ResponseError
from datetime import timedelta

#约定：s3包下面的所有文件均是对存储对象的操作，里面所有的变量，单词，描述，均是和存储对象相关的，不允许出现dataset相关的用语
class YcMinio():
    def __init__(self, access_key, secret_key, endpoint, secure=False):
        self.access_key = access_key
        self.secret_key = secret_key
        self.endpoint = endpoint.replace("http://", "").replace("https://", "")
        self.secure = secure

        self.client = Minio(
            self.endpoint,
            access_key = self.access_key,
            secret_key = self.secret_key,
            secure = False
        )


    #创建bucket
    def CreateBucket(self, bucketName, location='cn-north-1'):
        try:
            self.client.make_bucket(bucketName, location)
        except ResponseError as err:
            raise Exception("create bucket failed", err)

    #列出所有的Dataset
    def ListAllBucket(self):
        ret = []
        buckets = self.client.list_buckets()
        for bucket in buckets:
            ret.append(bucket.name)

        return ret

    #列出某个Dataset下面的所有文件
    def ListObjects(self, bucketName, prefix=None, recursive=False):
        ret = []
        objects = self.client.list_objects(bucketName, prefix, recursive)
        for obj in objects:
            # ret.append(obj.objectName.encode('utf-8'))
            ret.append(obj.object_name)

        return ret

    #上传单个文件数据
    def PutObject(self, bucketName, objectName, file_path, content_type='application/octet-stream', metadata=None):
        try:
            self.client.fput_object(bucketName, objectName, file_path)
        except ResponseError as err:
            raise Exception("failed in putting file", err)

    #删除单个文件
    def DeleteObject(self, bucketName, objectName):
        try:
            self.client.remove_object(bucketName, objectName)
        except ResponseError as err:
            raise Exception("failed in deleting file", err)

    #获取访问地址
    def PresignedGetObject(self, bucketName, objectName, expire=7):
        try:
            return self.client.presigned_get_object(bucketName, objectName, expires=timedelta(days=expire))
        except ResponseError as err:
            raise Exception("failed in create presigned object url", err)



import os
import json

# config 路径
def _config_filepath():
    home = "USERPROFILE" if os.name == "nt" else "HOME"
    configDir = os.path.join(os.environ[home], ".tda/")
    if not os.path.exists(configDir):
        os.makedirs(configDir)

    return os.path.join(configDir, "tda.conf")


# 未设置信息消息提示
def _noLoginMessage():
    print("您还未设置access_ke和secret_key, 请执行")
    print("\n")
    print("    tda config <access_key> <secret_key> [endpoint] 来设置您的tda")
    print("\n")
    print("关于tda config 更多帮助请执行：\n")
    print("    tda config --help")
    print("\n")
    exit()


# 检查登录
def _check():
    configFile = _config_filepath()
    if not os.path.exists(configFile):
        _noLoginMessage()

    with open(configFile, "r") as config:
        try:
            jsonObj = json.load(config)
        except:
            _noLoginMessage()

        if "access_key" not in jsonObj.keys():
            _noLoginMessage()

        if jsonObj["access_key"] == None or jsonObj["secret_key"] == None:
            _noLoginMessage()

        if jsonObj["access_key"] == "" or jsonObj["secret_key"] == "":
            _noLoginMessage()

    return True


# 获取conf
def _getConf():
    configFile = _config_filepath()
    if _check():
        with open(configFile, "r") as config:
            return json.load(config)



import os
import sys
import json
import re

sys.path.append(os.path.dirname(__file__) + os.sep + './')

from dataset.dataset import Dataset
from dataset.file import File
from utils import util
from dataset.localdb import LocalDb

#sdk入口
class TDA():
    def __init__(self, T_key):
        self.T_key = T_key
        self.commitFlag = False
        self.commitId = ""
        self.datasetName = ""
        self.fileList = []


    def SetDataset(self, datasetName, ip=""):
        if ip:
            trueIp = re.search(r'(([01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])\.){3}([01]{0,1}\d{0,1}\d|2[0-4]\d|25[0-5])', ip)
            self.ip = trueIp.group()

        self.datasetName = datasetName
        self.dataset = Dataset(self.T_key, datasetName, self.ip)

    def RiseException(self):
        raise Exception("You have not set the dataset name yet!")


    def ListAllFile(self, recu=False):
        if not self.datasetName:
            self.RiseException()

        return self.dataset.ListAllFile()


    def UploadFilesToDataset(self, rootPath, basepath="", ext="", batchNum=0):
        if not self.datasetName:
            self.RiseException()

        syncData = []
        j = 0
        i = 0
        retInfo = {'succ': 0, 'fail': 0}
        dirname = os.path.dirname(rootPath.rstrip("/"))
        for root, dir, files in os.walk(rootPath):
            for fileName in files:
                if fileName.endswith(ext):
                    filePath = os.path.join(root, fileName)
                    objectName = filePath.replace(dirname, "").lstrip("/")

                    if not basepath:
                        objectPath = objectName
                    else:
                        basepath = basepath.rstrip("/")
                        if filePath.find(basepath) < 0:
                            raise Exception(f"basepath must be part of filepath!")
                        objectPath = filePath.split(basepath + "/")[1]

                    self.__UploadFileToDataset(filePath, objectPath)
                    tmp = {
                        "ref_id": "",
                        "name": fileName,
                        "path": self.datasetName + "/" + objectPath,
                        "size": int(util.getFileSize(filePath)),
                        "md5": util.getFileMd5(filePath),
                        "meta": {},
                        "anotations": {},
                    }
                    syncData.append(tmp)
                    i += 1
                    j += 1

                    if j >= 1000:
                        j = 0
                        info = self.dataset.SyncDataToWeb(syncData)
                        retInfo["succ"] += info["succ"]
                        retInfo["fail"] += info["fail"]
                        syncData = []

        if len(syncData) >= 0:#最后几个没同步的数据
            info = self.dataset.SyncDataToWeb(syncData)
            retInfo["succ"] += info["succ"]
            retInfo["fail"] += info["fail"]

        return retInfo


    def UploadFileToDataset(self, filePath, basePath=""):
        basePath = basePath.rstrip("/")
        objectName = filePath.split(basePath)[1].lstrip("/")
        self.__UploadFileToDataset(filePath, objectName)
        syncData = {
            "ref_id": "",
            "name": os.path.basename(filePath),
            "path": self.datasetName + "/" + objectName,
            "size": int(util.getFileSize(filePath)),
            "md5": util.getFileMd5(filePath),
            "meta": {},
            "anotations": {},
        }

        info = self.dataset.SyncDataToWeb([syncData])

        return info


    def __UploadFileToDataset(self, filePath, objectName):
        if not self.datasetName:
            self.RiseException()
        return self.dataset.PutFileToDataset(objectName, filePath)

    def DeleteFileFromDataset(self, objectName):
        if not self.datasetName:
            self.RiseException()

        return self.dataset.DeleteFromDataset(objectName)

    def AddFile(self, filepath, basepath=""):
        if not self.datasetName:
            self.RiseException()

        file = File(filepath, basepath, self.datasetName)
        self.fileList.append(file)
        return file


    def Commit(self, commitId = ""):
        if not self.datasetName:
            self.RiseException()

        db = LocalDb(commitId)
        db.insertVal(self.fileList)
        db.close()

        self.commitFlag = True
        self.commitId = db.commitId
        return db.commitId


    def Upload(self, commitId = ""):
        if not self.datasetName:
            self.RiseException()

        if commitId != "":
            self.commitFlag = True
            self.commitId = commitId

        if not self.commitFlag:
            self.commitId = self.Commit(commitId)

        if not self.commitId:
            self.commitId = self.Commit(commitId)

        db = LocalDb(commitId)
        allData = db.fetchAll()

        syncData = []
        i = 0
        retInfo = {'succ': 0, 'fail': 0}
        for data in allData:
            self.__UploadFileToDataset(data["filepath"], data["objectPath"])
            tmp = {
                "ref_id":data["referid"],
                "name":data["filename"],
                "path":data["osspath"],
                "size":data["filesize"],
                "md5":data["md5"],
                "meta":json.loads(data["metadata"]),
                "anotations":json.loads(data["labeldata"]),
            }
            syncData.append(tmp)
            i += 1
            if i >= 1000:
                i = 0
                info = self.dataset.SyncDataToWeb(syncData)
                retInfo["succ"] += info["succ"]
                retInfo["fail"] += info["fail"]
                syncData = []


        if len(syncData) >= 0:#最后几个没同步的数据
            info = self.dataset.SyncDataToWeb(syncData)
            retInfo["succ"] += info["succ"]
            retInfo["fail"] += info["fail"]

        return retInfo

    def Delete(self, referId):
        if not referId:
            raise Exception("referId can not be empty")
        return self.dataset.Delete(referId)

    def GetData(self, offset=0, limit=100):
        if limit > 1000:
            raise Exception("limit must less than 1000")

        return self.dataset.GetData(offset, limit)









